// -------------------------------------------------------------------------
//    @FileName			:    NFCHeroPropertyModule.h
//    @Author           :    LvSheng.Huang
//    @Date             :    2017-02-16
//    @Module           :    NFCHeroPropertyModule
//
// -------------------------------------------------------------------------

#include "NFCHeroPropertyModule.h"

bool NFCHeroPropertyModule::Init()
{
	return true;
}

bool NFCHeroPropertyModule::Shut()
{
	return true;
}

bool NFCHeroPropertyModule::Execute()
{
	return true;
}

bool NFCHeroPropertyModule::AfterInit()
{
	m_pKernelModule = pPluginManager->FindModule<NFIKernelModule>();
	m_pLogicClassModule = pPluginManager->FindModule<NFIClassModule>();
	m_pGameServerNet_ServerModule = pPluginManager->FindModule<NFIGameServerNet_ServerModule>();
	m_pElementModule = pPluginManager->FindModule<NFIElementModule>();
	m_pEquipPropertyModule = pPluginManager->FindModule<NFIEquipPropertyModule>();
	m_pPropertyModule = pPluginManager->FindModule<NFIPropertyModule>();
	m_pHeroModule = pPluginManager->FindModule<NFIHeroModule>();
	
	m_pKernelModule->AddClassCallBack(NFrame::Player::ThisName(), this, &NFCHeroPropertyModule::OnPlayerClassEvent);

	return true;

}

bool NFCHeroPropertyModule::BeforeShut()
{
	return true;
}

int NFCHeroPropertyModule::OnPlayerClassEvent(const NFGUID& self, const std::string& strClassName, const CLASS_OBJECT_EVENT eClassEvent, const NFDataList& var)
{
	switch (eClassEvent)
	{
	case CLASS_OBJECT_EVENT::COE_CREATE_BEFORE_EFFECT:
	{
		m_pKernelModule->AddRecordCallBack(self, NFrame::Player::PlayerHero::ThisName(), this, &NFCHeroPropertyModule::OnObjectHeroRecordEvent);
		CalFightintHeroProperty(self);
	}
	break;
	case CLASS_OBJECT_EVENT::COE_CREATE_FINISH:
	{
	}
	break;
	default:
		break;
	}

	return 0;
}

int NFCHeroPropertyModule::OnObjectHeroRecordEvent(const NFGUID& self, const RECORD_EVENT_DATA& xEventData, const NFData& oldVar, const NFData& newVar)
{
	std::ostringstream stream;
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, xEventData.strRecordName);
	if (nullptr == pHeroRecord)
	{
		return 0;
	}

	switch (xEventData.nOpType)
	{
	case RECORD_EVENT_DATA::Add:
	{
		const NFGUID xHeroID = pHeroRecord->GetObject(xEventData.nRow, NFrame::Player::PlayerHero::GUID);
		
		AddHeroProperty(self, xHeroID);

		if (m_pKernelModule->GetPropertyObject(self, NFrame::Player::HeroID1()) == NFGUID())
		{
			//must setting props before we set props for fighting hero
			m_pHeroModule->SetFightHero(self, xHeroID, NFIHeroModule::EConsHero_Pos::ECONSt_HERO_POS1);
		}
		else if (m_pKernelModule->GetPropertyObject(self, NFrame::Player::HeroID2()) == NFGUID())
		{
			m_pHeroModule->SetFightHero(self, xHeroID, NFIHeroModule::EConsHero_Pos::ECONSt_HERO_POS2);
		}
		else if (m_pKernelModule->GetPropertyObject(self, NFrame::Player::HeroID3()) == NFGUID())
		{
			m_pHeroModule->SetFightHero(self, xHeroID, NFIHeroModule::EConsHero_Pos::ECONSt_HERO_POS3);
		}

		const NFGUID xFightingHeroID = m_pKernelModule->GetPropertyObject(self, NFrame::Player::FightHero());
		if (xFightingHeroID == xHeroID)
		{
			CalFightintHeroProperty(self);
		}
	}
	break;
	case RECORD_EVENT_DATA::Del:
	{

	}
	break;
	case RECORD_EVENT_DATA::Update:
	{
		switch (xEventData.nCol)
		{
		case NFrame::Player::PlayerHero::Level:
		case NFrame::Player::PlayerHero::Star:
		{
			const NFGUID& xHeroGUID = pHeroRecord->GetObject(xEventData.nRow, NFrame::Player::PlayerHero::GUID);
			OnHeroPropertyUpdate(self, xHeroGUID);
		}
		break;
		default:
			break;
		}
	}
	break;
	default:
		break;
	}

	return 0;
}

bool NFCHeroPropertyModule::OnHeroPropertyUpdate(const NFGUID & self, const NFGUID & xHeroGUID)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);

	NFDataList xHeroAllValue;
	bool bRet = CalHeroAllProperty(self, xHeroGUID, xHeroAllValue);
	if (bRet)
	{
		if (pHeroPropertyRecord->IsUsed(nRow))
		{
			pHeroPropertyRecord->SetRow(nRow, xHeroAllValue);
		}
		else
		{
			pHeroPropertyRecord->AddRow(nRow, xHeroAllValue);
		}
	}

	int nStar = pHeroRecord->GetInt(nRow, NFrame::Player::PlayerHero::Star);
	NFIHeroModule::EConsHero_Pos ePos = m_pHeroModule->GetFightPos(self, xHeroGUID);
	switch (ePos)
	{
	case NFIHeroModule::ECONSt_HERO_UNKNOW:
		break;
	case NFIHeroModule::ECONSt_HERO_POS1:
	{
		m_pKernelModule->SetPropertyInt(self, NFrame::Player::HeroPos1Star(), nStar);
	}
		break;
	case NFIHeroModule::ECONSt_HERO_POS2:
	{
		m_pKernelModule->SetPropertyInt(self, NFrame::Player::HeroPos2Star(), nStar);
	}
		break;
	case NFIHeroModule::ECONSt_HERO_POS3:
	{
		m_pKernelModule->SetPropertyInt(self, NFrame::Player::HeroPos3Star(), nStar);
	}
		break;
	case NFIHeroModule::ECONSt_HERO_MAX:
		break;
	default:
		break;
	}

	NFGUID xFightingID = m_pKernelModule->GetPropertyObject(self, NFrame::Player::FightHero());
	if (xFightingID == xHeroGUID)
	{
		CalFightintHeroProperty(self);
	}

	return true;
}

bool NFCHeroPropertyModule::AddHeroProperty(const NFGUID & self, const NFGUID & xHeroGUID)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);

	NFDataList xHeroAllValue;
	bool bRet = CalHeroAllProperty(self, xHeroGUID, xHeroAllValue);
	if (bRet)
	{
		if (pHeroPropertyRecord->IsUsed(nRow))
		{
			pHeroPropertyRecord->SetRow(nRow, xHeroAllValue);
		}
		else
		{
			pHeroPropertyRecord->AddRow(nRow, xHeroAllValue);
		}
	}

	return true;
}

bool NFCHeroPropertyModule::CalHeroAllProperty(const NFGUID& self, const NFGUID& xHeroGUID, NFDataList& xDataList)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	//const int nRow = varFind.Int32(0);

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	NFDataList xHeroPropertyValue;
	CalHeroBaseProperty(self, xHeroGUID, xHeroPropertyValue);

	NFDataList xHeroTalentValue;
	CalHeroTalentProperty(self, xHeroGUID, xHeroTalentValue);

	NFDataList xHeroEquipValue;
	CalHeroEquipProperty(self, xHeroGUID, xHeroEquipValue);

	for (int i = 0; i < pHeroPropertyRecord->GetCols(); ++i)
	{
		int nPropertyValue = 0;
		int nTalentValue = 0;
		int nEquipValue = 0;

		if (xHeroPropertyValue.GetCount() == pHeroPropertyRecord->GetCols())
		{
			nPropertyValue = xHeroPropertyValue.Int32(i);
		}
		if (xHeroTalentValue.GetCount() == pHeroPropertyRecord->GetCols())
		{
			nTalentValue = xHeroTalentValue.Int32(i);
		}
		if (xHeroEquipValue.GetCount() == pHeroPropertyRecord->GetCols())
		{
			nEquipValue = xHeroEquipValue.Int32(i);
		}

		int nAllValue = nPropertyValue + nTalentValue + nEquipValue;

		xDataList.AddInt(nAllValue);
	}

	return true;
}

bool NFCHeroPropertyModule::CalHeroBaseProperty(const NFGUID& self, const NFGUID& xHeroGUID, NFDataList& xDataList)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	/////////////PropertyBase/////////////////////////////////////////
	xDataList.Clear();
	
	const int nHeroStar = pHeroRecord->GetInt(nRow, NFrame::Player::PlayerHero::Star);
	const std::string& strConfigID = pHeroRecord->GetString(nRow, NFrame::Player::PlayerHero::ConfigID);
	const std::string& strPropertyEffectData = m_pElementModule->GetPropertyString(strConfigID, NFrame::NPC::EffectData());

	int nLeftStar = 0;
	const int nDiamond = CalHeroDiamond(nHeroStar, nLeftStar);
	float fPropsTime = 1.0f;
	fPropsTime += (nDiamond + nLeftStar / float(5 - nDiamond + 1));

	if (!strPropertyEffectData.empty())
	{
		for (int i = 0; i < pHeroPropertyRecord->GetCols(); ++i)
		{
			const std::string& strColTag = pHeroPropertyRecord->GetColTag(i);
			int64_t nValue = m_pElementModule->GetPropertyInt(strPropertyEffectData, strColTag);

			switch (i)
			{
			case NFrame::Player::CommValue::MAXHP:
			case NFrame::Player::CommValue::MAXMP:
			case NFrame::Player::CommValue::MAXSP:
			case NFrame::Player::CommValue::ATK_VALUE:
			case NFrame::Player::CommValue::DEF_VALUE:
			case NFrame::Player::CommValue::HPREGEN:
			case NFrame::Player::CommValue::MPREGEN:
			{
				nValue = nValue * fPropsTime;
			}
			default:
				break;
			}

			xDataList.AddInt(nValue);
		}
	}

	return true;
}

bool NFCHeroPropertyModule::CalHeroTalentProperty(const NFGUID& self, const NFGUID& xHeroGUID, NFDataList& xDataList)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	////////////////////Talent//////////////////////////////////////////////////////
	xDataList.Clear();

	for (int i = 0; i < pHeroPropertyRecord->GetCols(); ++i)
	{
		xDataList.AddInt(0);
	}
	/*
	for (int i = NFrame::Player::PlayerHero::Talent1; i <= NFrame::Player::PlayerHero::Talent5; ++i)
	{
		const std::string& strTalentID = pHeroRecord->GetString(nRow, i);
		const std::string& strTalentEffectData = m_pElementModule->GetPropertyString(strTalentID, NFrame::Talent::EffectData());
		if (!strTalentEffectData.empty())
		{
			//one talent
			for (int j = 0; j < pHeroPropertyRecord->GetCols(); ++j)
			{
				const std::string& strColTag = pHeroPropertyRecord->GetColTag(j);
				int64_t nValue = m_pElementModule->GetPropertyInt(strTalentEffectData, strColTag);
				int64_t nOldValue = xDataList.Int(j);

				xDataList.SetInt(j, nOldValue + nValue);
			}
		}
	}
	*/
	return true;
}

bool NFCHeroPropertyModule::CalHeroEquipProperty(const NFGUID& self, const NFGUID& xHeroGUID, NFDataList& xDataList)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	////////////////////Equip//////////////////////////////////////////////////////
	xDataList.Clear();

	for (int i = 0; i < pHeroPropertyRecord->GetCols(); ++i)
	{
		xDataList.AddInt(0);
	}
	/*
	for (int i = NFrame::Player::PlayerHero::Equip1; i <= NFrame::Player::PlayerHero::Equip6; ++i)
	{
		NFDataList EquipDataList;
		const NFGUID xEquipID = pHeroRecord->GetObject(nRow, i);
		if (!xEquipID.IsNull() && m_pEquipPropertyModule->CalEquipProperty(self, xEquipID, EquipDataList))
		{
			//one equip
			for (int j = 0; j < pHeroPropertyRecord->GetCols(); ++j)
			{
				int64_t nOldValue = xDataList.Int(j);
				int64_t nEquipValue = EquipDataList.Int(j);

				xDataList.SetInt(j, nOldValue + nEquipValue);
			}
		}
	}
	*/
	return true;
}

bool NFCHeroPropertyModule::CalFightintHeroProperty(const NFGUID & self)
{
	NF_SHARE_PTR<NFIRecord> pHeroRecord = m_pKernelModule->FindRecord(self, NFrame::Player::PlayerHero::ThisName());
	if (nullptr == pHeroRecord)
	{
		return false;
	}
	NFGUID xHeroGUID = m_pKernelModule->GetPropertyObject(self, NFrame::Player::FightHero());
	if (xHeroGUID.IsNull())
	{
		return false;
	}

	NFDataList varFind;
	if (pHeroRecord->FindObject(NFrame::Player::PlayerHero::GUID, xHeroGUID, varFind) != 1)
	{
		return false;
	}

	NF_SHARE_PTR<NFIRecord> pHeroPropertyRecord = m_pKernelModule->FindRecord(self, NFrame::Player::HeroValue::ThisName());
	if (nullptr == pHeroPropertyRecord)
	{
		return false;
	}

	const int nRow = varFind.Int32(0);
	for (int i = 0; i < pHeroPropertyRecord->GetCols(); ++i)
	{
		const std::string& strColTag = pHeroPropertyRecord->GetColTag(i);
		int64_t nValue = pHeroPropertyRecord->GetInt(nRow, i);

		m_pPropertyModule->SetPropertyValue(self, strColTag, NFIPropertyModule::NPG_FIGHTING_HERO, nValue);
	}


	return true;
}

int NFCHeroPropertyModule::CalHeroDiamond(const int star, int & nStar)
{
	if (star < 0 || star >= NFIHeroModule::EConstDefine_Hero::ECONSTDEFINE_HERO_MAX_STAR)
	{
		nStar = 0;
		return 0;
	}

	int nHeroDia = 0;
	int nHeroStar = 0;

	if (star <= 5)
	{
		nHeroDia = 0;
		nHeroStar = star;
	}
	else if (star <= 10)
	{
		nHeroDia = 1;
		nHeroStar = star - 6;
	}
	else if (star <= 14)
	{
		nHeroDia = 2;
		nHeroStar = star - 11;
	}
	else if (star <= 17)
	{
		nHeroDia = 3;
		nHeroStar = star - 15;
	}
	else if (star <= 19)
	{
		nHeroDia = 4;
		nHeroStar = star - 18;
	}
	else if (star <= 20)
	{
		nHeroDia = 5;
		nHeroStar = 0;
	}

	nStar = nHeroStar;

	return nHeroDia;
}
